from typing import Callable, Optional
from uuid import UUID

from fractal_specifications.generic.specification import Specification
from pydantic.main import BaseModel


class TokenPayload(BaseModel):
    iss: Optional[str]
    sub: Optional[UUID]
    account: Optional[UUID]
    email: Optional[str]
    typ: Optional[str]


class TokenPayloadRoles(TokenPayload):
    roles: Optional[list]
    specification_func: Callable = lambda **kwargs: None

    @property
    def specification(self) -> Optional[Specification]:
        if self.specification_func:
            return self.specification_func(**self.dict())



from dataclasses import dataclass
from typing import Callable, Optional

from fractal_specifications.generic.operators import EqualsSpecification
from fractal_specifications.generic.specification import Specification


class Role:
    def __getattr__(self, item):
        return Methods()

    @staticmethod
    def filter_account(**kwargs) -> Specification:
        return EqualsSpecification("account_id", str(kwargs.get("account")))


@dataclass
class Method:
    specification_func: Callable = lambda **kwargs: None


class Methods:
    get: Optional[Method]
    post: Optional[Method]
    put: Optional[Method]
    delete: Optional[Method]

    def __init__(self, method: Method = None, **kwargs):
        if not method:
            method = Method(lambda **kwargs: Role.filter_account(**kwargs))
        self.get = kwargs["get"] if "get" in kwargs else method
        self.post = kwargs["post"] if "post" in kwargs else method
        self.put = kwargs["put"] if "put" in kwargs else method
        self.delete = kwargs["delete"] if "delete" in kwargs else method



from typing import Optional

from fractal import Fractal
from fractal.contrib.roles.models import Methods, Role
from fractal.contrib.roles.services import BaseRolesService
from fractal.core.utils.application_context import ApplicationContext
from fractal.core.utils.settings import Settings


class Admin(Role):
    def __getattr__(self, item):
        return Methods()


class DummyRolesService(BaseRolesService):
    def __init__(self):
        self.roles = [Admin()]


class DummyTokenRolesFractal(Fractal):
    def __init__(self, context: ApplicationContext, settings: Optional[Settings]):
        self.context = context
        self.token_service = getattr(context, "token_service")
        if roles_service := getattr(context, "roles_service", None):
            self.roles_service = roles_service
        else:
            context.install_service(DummyRolesService, name="roles_service")
        self.settings = settings
        super(DummyTokenRolesFractal, self).__init__()



from typing import List

from fractal.contrib.roles.models import Role
from fractal.contrib.tokens.exceptions import NotAllowedException
from fractal.contrib.tokens.models import TokenPayloadRoles
from fractal.core.services import Service


class BaseRolesService(Service):
    roles: List[Role] = []

    def verify(
        self, payload: TokenPayloadRoles, endpoint: str, method: str
    ) -> TokenPayloadRoles:
        for role in self.roles:
            if role.__class__.__name__.lower() in payload.roles:
                if e := getattr(role, endpoint, None):
                    if getattr(e, method, None):
                        if m := getattr(e, method, None):
                            payload.specification_func = m.specification_func
                        return payload
        raise NotAllowedException("No permission!")
